type javaType = 
  JavaVoid 
| JavaBool 
| JavaInt 
| JavaFloat 
| JavaString 
| JavaInvalidType 
| JavaCollection
| JavaEnumItem 
| JavaEnum
| JavaStruct
| Indeterminate

type identifier = 
	Identifier of string

type functionId =
	FunctionIdentifier of string * identifier list

type expression = 
	Integer of int
|	Boolean of bool
|	Floating of float
|	StringString of string
|	IdExpr of identifier
|   Collection of expression list
|   CollectionBounds of expression * expression
|	Enum of identifier list
|	ParenExpr of expression
|	Struct of structDef list
|	Cast of expression * string
|   Input of string
|	Negate of expression
|   Invert of expression
|	BinaryOp of binaryOperation
|	Increment of expression
|   Decrement of expression
|	CollectionIndex of identifier * expression
|	Lookup of identifier * expression
|	FunctionCall of functionId

and structDef = 
	StructDef of string * expression

and binaryOperation =
	Plus of expression * expression
|	Minus of expression * expression
|	Times of expression * expression
|	Divide of expression * expression
|	Mod of expression * expression
|	And of expression * expression
|	Or of expression * expression
|	Neq of expression * expression
|	Eq of expression * expression
|	Lt of expression * expression
|	Gt of expression * expression
|	Leq of expression * expression
|	Geq of expression * expression

type statement = 
	Expression of expression
|	LocalVar of assignment	
|   Update of update
|	Output of expression
|	Return of expression
|	IfStructure of ifStatement list
|	WhileStructure of expression * block
|	RuleStructure of assignment list * ruleBlock list
|	ForStructure of identifier * expression * block

and block =
	Block of statement list

and ruleBlock =
	RuleBlock of expression * block

and assignment = 
	Assignment of identifier * expression
|	FunctionAssign of functionId * block
|	Empty

and update = 
	PlusUp of identifier * expression
|	MinusUp of identifier * expression
|	TimesUp	of identifier * expression
|	DivideUp of identifier * expression

and ifStatement = 
	IfStatement of string * expression * block

type program =
	assignment list
